/****************************************************************************
 Module
   BeaconService.c

 Revision
   1.0.1

 Description
   This is a template file for implementing the Beacon Service that detects the
 * one IR signal using two photo-transistors and determines alignment. 

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 01/16/12 09:58 jec      began conversion from TemplateFSM.c
 ****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
 */
#include "ES_Configure.h"
#include "ES_Framework.h"
#include "PIC32_AD_Lib.h"
#include "PIC32_Port_HAL.h"
#include "PIC32_SPI_HAL.h"
#include "EyeColor.h"
#include "LeaderService.h"

//to make ISRs
#include <xc.h>
#include <sys/attribs.h>

/*----------------------------- Module Defines ----------------------------*/
//frequency at which alignment is checked and sent to follower PIC
#define ALIGN_TIME 100
#define MISALIGN_TIME 10
#define MISALIGN_MODULO 5

// if beaconTime hasn't been updated in this long, assume we are misaligned and 
// send misalign event to follower

//conversion from ticks, to microseconds, multiply by this
#define CONVERT_TICKS_MICRO 12.8 
#define MEDIAN_LENGTH 4

#define FREQ_TOL 100
#define FREQ_TEAM_A 3333
#define FREQ_TEAM_B 909
/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this service.They should be functions
   relevant to the behavior of this service
 */
static void InitTmr3AndIC3(void);
static void ClearBeaconTimes();
static uint32_t MedianBeaconTimes();
/*---------------------------- Module Variables ---------------------------*/
// with the introduction of Gen2, we need a module level Priority variable
static uint8_t MyPriority;

static uint32_t AnalogData[] = {0};
static uint32_t Right;
static uint32_t Left;
static bool wasAligned, isRed, isBlue, firstAlignmentDone;
static uint32_t beaconTime;
static float beaconFreq;
static uint32_t beaconTimes[MEDIAN_LENGTH];
static uint32_t beaconTimeIndex;
static uint16_t lastBeaconAlignTime;
static uint16_t beaconMisalignIndex;
static uint32_t lastRiseTime;

volatile static union FeatureWidthTime {

    struct {
        uint16_t IC;
        uint16_t Rollover;
    } ByBytes;
    uint32_t Time;
} RiseTime;
/*------------------------------ Module Code ------------------------------*/

/****************************************************************************
 Function
     InitTemplateService

 Parameters
     uint8_t : the priorty of this service

 Returns
     bool, false if error in initialization, true otherwise

 Description
     Saves away the priority, and does any
     other required initialization for this service
 Notes

 Author
     J. Edward Carryer, 01/16/12, 10:00
 ****************************************************************************/
bool InitBeaconService(uint8_t Priority) {
    ES_Event_t ThisEvent;

    MyPriority = Priority;

    wasAligned = false;
    lastBeaconAlignTime = 0;
    beaconTimeIndex = 0;
    beaconMisalignIndex = 0;
    ClearBeaconTimes();
    isRed = 0;
    isBlue = 0;
    firstAlignmentDone = false;

    printf("Initializing Beacon Service...\r\n");

    //  Right = 0;
    //  Left = 0;

    // RB12 = right, RB13 = left analog inputs
    //  PortSetup_ConfigureAnalogInputs(_Port_B, _Pin_12);
    //  PortSetup_ConfigureAnalogInputs(_Port_B, _Pin_13);
    // AnalogData[0] = left, AnalogData[1] = right
    //  ADC_ConfigAutoScan(1 << 11 | 1 << 12, 2); 

    // Setup TMR3 and IC3 for input capture on the digital signal from beacon
    InitTmr3AndIC3();

    /********************************************
     in here you write your initialization code
     *******************************************/
    // post the initial transition event
    ThisEvent.EventType = ES_INIT;
    if (ES_PostToService(MyPriority, ThisEvent) == true) {
        return true;
    } else {
        return false;
    }
}

/****************************************************************************
 Function
     PostTemplateService

 Parameters
     EF_Event_t ThisEvent ,the event to post to the queue

 Returns
     bool false if the Enqueue operation failed, true otherwise

 Description
     Posts an event to this state machine's queue
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:25
 ****************************************************************************/
bool PostBeaconService(ES_Event_t ThisEvent) {
    return ES_PostToService(MyPriority, ThisEvent);
}

/****************************************************************************
 Function
    RunTemplateService

 Parameters
   ES_Event_t : the event to process

 Returns 
   ES_Event, ES_NO_EVENT if no error ES_ERROR otherwise

 Description
   add your description here
 Notes

 Author
   J. Edward Carryer, 01/15/12, 15:23
 ****************************************************************************/
ES_Event_t RunBeaconService(ES_Event_t ThisEvent) {
    ES_Event_t ReturnEvent;
    ReturnEvent.EventType = ES_NO_EVENT; // assume no errors
    /********************************************
     in here you write your service code
     *******************************************/
    switch (ThisEvent.EventType) {
        case ES_INIT:
            printf("beacon starts out misaligned\r\n");
        case ES_TIMEOUT:
        {
            // Timer that allows us to check if it's been awhile since beaconTime 
            // was updated, in which case it has been misaligned
            uint16_t time_t = ES_Timer_GetTime() - lastBeaconAlignTime;

            if (time_t > MISALIGN_TIME) {
                ES_Event_t MisalignEvent;
                if (((beaconMisalignIndex % MISALIGN_MODULO) == 0) || wasAligned) {
                    wasAligned = false;
                    beaconFreq = 0;

                    // send misalign event to leader
                    printf("misaligned! beacon freq: %f\r\n", beaconFreq);
                    MisalignEvent.EventType = HW_MISALIGNED;
                    MisalignEvent.EventParam = 0;
                    PostLeaderService(MisalignEvent);
                }
                beaconMisalignIndex++;
            }

            ES_Timer_InitTimer(BEACON_TIMER, ALIGN_TIME);
        }
        break;
        case HW_ALIGNED:
        {
            beaconFreq = (float) 1e6 / ThisEvent.EventParam;


            // only change team color if this is first alignment
            if (!firstAlignmentDone) {

                isRed = (beaconFreq > (FREQ_TEAM_A - FREQ_TOL)) && (beaconFreq < (FREQ_TEAM_A + FREQ_TOL));
                isBlue = (beaconFreq > (FREQ_TEAM_B - FREQ_TOL)) && (beaconFreq < (FREQ_TEAM_B + FREQ_TOL));

                if (isRed || isBlue) {

                    firstAlignmentDone = true;
                    ChangeEyeColor(isBlue);
                    if (isRed) {
                        printf("RED\r\n");
                    } else {
                        printf("BLUE\r\n");
                    }
                }
                printf("first alignment\r\n");
            }
            printf("aligned! beacon freq: %f\r\n", beaconFreq);

            // only consider aligned if detecting the frequency for the correct
            // team
            bool isCurrentlyRed = (beaconFreq > (FREQ_TEAM_A - FREQ_TOL)) && (beaconFreq < (FREQ_TEAM_A + FREQ_TOL));
            bool isCurrentlyBlue = (beaconFreq > (FREQ_TEAM_B - FREQ_TOL)) && (beaconFreq < (FREQ_TEAM_B + FREQ_TOL));
            if ((isRed && isCurrentlyRed) || (isBlue && isCurrentlyBlue)) {
                wasAligned = true;
                beaconMisalignIndex = 0;
                // send to main leader
                ES_Event_t BeaconEvent;
                BeaconEvent.EventType = HW_BEACON;
                PostLeaderService(BeaconEvent);
            }

        }
        break;
            //      case HW_MISALIGNED:
            //          beaconFreq = (float) 1e6 / ThisEvent.EventParam;
            //          printf("misaligned! beacon freq: %f\r\n", beaconFreq);
            //          
            //          // send to main leader service
            //          ES_Event_t BeaconEvent;
            //          BeaconEvent.EventType = HW_MISALIGNED;
            //          PostLeaderService(BeaconEvent);
            //          
            //          break;
    }
    return ReturnEvent;
}

/***************************************************************************
 private functions
 ***************************************************************************/
static void InitTmr3AndIC3(void) {
    //This section sets up the timer 3 bcs want Input Capture//  
    //clear the ON control bits (T3CON<15>) to disable timer
    T3CONbits.ON = 0;
    //clear the TCS control bit (T3CON<1>) to select internal PBCLK clock source
    T3CONbits.TCS = 0;
    //select prescale 265
    T3CONbits.TCKPS = 0b111; //265 111 //32 101
    //load the period register PR3
    PR3 = 0xFFFF;
    //clear the T2IF interrupt flag in the IFS0 register (write to IFS0CLR)
    IFS0CLR = _IFS0_T3IF_MASK;
    //timer interrupts needed to count TMR3 rollover
    //configure interrupt priority and subpriority in the IPC3 register (IPC3bits.T3IP = 6)
    IPC3bits.T3IP = 6;
    //set the T3IE interrupt enable bit in the IEC0 register (IEC0SET = _IEC0_T3IE_MASK)
    IEC0SET = _IEC0_T3IE_MASK;
    //clear timer register TMR3
    TMR3 = 0;
    //set the ON control bit to enable timer
    T3CONbits.ON = 1;

    //This section sets up the input capture with IC3CON//
    //disable IC3 (IC3CONbits.ON = 0)
    IC3CONbits.ON = 0;
    //set continue to operate in idel mode
    IC3CONbits.SIDL = 0;
    //set First Capture Edge Select (FEDGE) bit to capture falling edge first (0), so can keep track of bits
    IC3CONbits.FEDGE = 0;
    //set 16-bit timer resource (C32 = 0)
    IC3CONbits.C32 = 0;
    //select timer 3 with ICTMR = 0
    IC3CONbits.ICTMR = 0;
    //set to interrupt on every capture event (ICI = 0)
    IC3CONbits.ICI = 0;
    //set input capture mode to ONLY rising edges
    IC3CONbits.ICM = 0b011;
    //set interrupt priority and subpriority level (IPC3bits.IC3IP = 7)
    IPC3bits.IC3IP = 7;
    //clear any pending masks
    IFS0CLR = _IFS0_IC3IF_MASK;
    //enable interrupt
    IEC0SET = _IEC0_IC3IE_MASK;
    //re-enable IC3 (IC3CONbits.ON = 1)
    IC3CONbits.ON = 1;

    //This section maps the pin that will receive the signal and map it//    
    //clear the ANSEL bit to disable analog on the pin **the Morse signal has 
    PortSetup_ConfigureDigitalInputs(_Port_B, _Pin_5);
    //been processed with a hysterisis circuit
    //configure the specified pin as Digital Input using TRISx
    // If a data direction bit is 1, the corresponding I/O port pin is an input
    //    TRISBSET = 1 << 5;
    //map the pin RB5 to IC3R (IC3R = 0b0001)
    IC3R = 0b0001; //0001 for RB5

}//End of InitButtonStatus

/**ISR Timer3 Function**/
void __ISR(_TIMER_3_VECTOR, IPL6SOFT) Timer3Handler(void) { // NOTE: Soft must match priority given
    //Disable interrupts globally 
    EnterCritical();

    if (IFS0bits.T3IF == 1) {//If T3IF is pending (IC ISR has NOT fired and handled already)
        //Increment roll-over counter
        ++RiseTime.ByBytes.Rollover;
        //clear roll-over interrupt IEC0bits.T
        IFS0CLR = _IFS0_T3IF_MASK;
    }
    //Enable interrupts 
    ExitCritical();
}

//Input Capture ISR

void __ISR(_INPUT_CAPTURE_3_VECTOR, IPL7SOFT) IC3Handler(void) {
    do {
        RiseTime.ByBytes.IC = (uint16_t) IC3BUF;
        if (IFS0bits.T3IF == 1) {
            if (RiseTime.ByBytes.IC < 0x8000) {
                //Increment the roll over counter TimeofLastFall.Rollover++
                RiseTime.ByBytes.Rollover = RiseTime.ByBytes.Rollover + 1;
                //clear the roll-over interrupt IFS0CLR w/ the Timer 3 mask _IFS0_T3IF_Mask
                IFS0CLR = _IFS0_T3IF_MASK;
            }
        }

        lastBeaconAlignTime = ES_Timer_GetTime();

        beaconTime = (RiseTime.Time - lastRiseTime) * CONVERT_TICKS_MICRO;
        beaconTimeIndex = beaconTimeIndex % MEDIAN_LENGTH;
        beaconTimes[beaconTimeIndex] = beaconTime;
        if (beaconTimeIndex++ == (MEDIAN_LENGTH - 1)) {
            if (!wasAligned) {
                uint32_t medianBeaconTime = MedianBeaconTimes();
                ClearBeaconTimes();

                ES_Event_t Beacon;
                Beacon.EventType = HW_ALIGNED;
                Beacon.EventParam = medianBeaconTime;
                PostBeaconService(Beacon);
            }
        }
        lastRiseTime = RiseTime.Time;

    } while (IC3CONbits.ICBNE != 0);

    //set direction equal to 0 to prep for next falling edge
    //clear the capture interrupt
    IFS0CLR = _IFS0_IC3IF_MASK;
}

static void ClearBeaconTimes() {
    for (int i = 0; i < MEDIAN_LENGTH; i++) {
        beaconTimes[i] = 0;
    }
}

static uint32_t MedianBeaconTimes() {
    int i = 0, j = 0, temp = 0;

    // sort beaconTimes
    for (i = 0; i < MEDIAN_LENGTH; i++) {
        for (j = 0; j < MEDIAN_LENGTH - 1; j++) {
            if (beaconTimes[j] > beaconTimes[j + 1]) {
                temp = beaconTimes[j];
                beaconTimes[j] = beaconTimes[j + 1];
                beaconTimes[j + 1] = temp;
            }
        }
    }

    //    for(i = 0; i < MEDIAN_LENGTH; i++)
    //    {
    //        printf("beaconFreqs[%d] : %f\r\n", i, (float) 1e6 / beaconTimes[i]);
    //    }

    // find median of beaconTimes
    uint32_t median = 0;

    // if number of elements are even
    if (MEDIAN_LENGTH % 2 == 0)
        median = (beaconTimes[(MEDIAN_LENGTH - 1) / 2] + beaconTimes[MEDIAN_LENGTH / 2]) / 2;
        // if number of elements are odd
    else
        median = beaconTimes[MEDIAN_LENGTH / 2];

    return median;
}
/*------------------------------- Footnotes -------------------------------*/
/*------------------------------ End of file ------------------------------*/

