/****************************************************************************
 Module
   Buttons.c

 Revision
   1.0.1

 Description
   This is a module that contains init function and event checkers for the 
 * dunk/load buttons.

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 01/15/12 11:12 jec      revisions for Gen2 framework
 11/07/11 11:26 jec      made the queue static
 10/30/11 17:59 jec      fixed references to CurrentEvent in RunTemplateSM()
 10/23/11 18:20 jec      began conversion from SMTemplate.c (02/20/07 rev)
 ****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
 */
#include "ES_Configure.h"
#include "ES_Framework.h"
#include "PIC32_Port_HAL.h"
#include "TestHarnessService.h"
#include "LeaderService.h"
#include "Buttons.h"

/*----------------------------- Module Defines ----------------------------*/
/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine.They should be functions
   relevant to the behavior of this state machine
 */

/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well.
// type of state variable should match htat of enum in header file
static uint8_t LastLoadButtonState;

/*------------------------------ Module Code ------------------------------*/

/****************************************************************************
 Function
     InitButtons

 Parameters
     uint8_t : the priorty of this service

 Returns
     bool, false if error in initialization, true otherwise

 Description
     Saves away the priority, sets up the initial transition and does any
     other required initialization for this state machine
 Notes

 Author
    Kathy Huang 11/06/21
 ****************************************************************************/
void InitButtons(void) {
    PortSetup_ConfigureDigitalInputs(_Port_B, _Pin_9);
    PortSetup_ConfigurePullDowns(_Port_B, _Pin_9);
    LastLoadButtonState = 0;
}

/****************************************************************************
 Function
    CheckLoadButtonPress

 Parameters
 * None

 Returns
 * bool True if the button is pressed, False if not

 Description
 * Posts HW_BUTTONPRESS event everytime a rising edge is detected
 Notes

 Author
    Kathy Huang 10/25/2021
 ****************************************************************************/
bool CheckLoadButtonPress(void) {
    bool returnVal = false;
    uint8_t CurrentLoadButtonState = PORTBbits.RB9; // read current button value
    // if the button is currently 1, then it has been pressed down
    if (CurrentLoadButtonState == 1 && CurrentLoadButtonState != LastLoadButtonState) {
        returnVal = true;
        ES_Event_t newButtonPress;
        newButtonPress.EventType = HW_BUTTONPRESS;
        newButtonPress.EventParam = LOAD;
        PostTestHarnessService(newButtonPress);

        ES_Event_t enterLoadMode;
        enterLoadMode.EventType = HW_LOADMODE;
        PostLeaderService(enterLoadMode);
    }
    LastLoadButtonState = CurrentLoadButtonState;

    return returnVal;
}

/****************************************************************************
 Function
    CheckDunkButtonPress

 Parameters
 * None

 Returns
 * bool True if the button is pressed, False if not

 Description
 * Posts HW_BUTTONPRESS event everytime a rising edge is detected
 Notes

 Author
    Kathy Huang 10/25/2021
 ****************************************************************************/
bool CheckDunkButtonPress(void) {
    bool returnVal = false;
    uint8_t CurrentDunkButtonState = PORTBbits.RB10; // read current button value
    // if the button is currently 1, then it has been pressed down
    if (CurrentDunkButtonState == 1 && CurrentDunkButtonState != LastDunkButtonState) {
        returnVal = true;
        ES_Event_t newButtonPress;
        newButtonPress.EventType = HW_BUTTONPRESS;
        newButtonPress.EventParam = DUNK;
        PostTestHarnessService(newButtonPress);

        ES_Event_t enterDunkMode;
        enterDunkMode.EventType = HW_DUNKMODE;
        PostLeaderService(enterDunkMode);
    }
    LastDunkButtonState = CurrentDunkButtonState;
    return returnVal;
}


/***************************************************************************
 private functions
 ***************************************************************************/

