/****************************************************************************
 Module
   DunkButtonService.c

 Revision
   1.0.1

 Description
   This is a module for implementing the dunk button switch debounce

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 01/15/12 11:12 jec      revisions for Gen2 framework
 11/07/11 11:26 jec      made the queue static
 10/30/11 17:59 jec      fixed references to CurrentEvent in RunTemplateSM()
 10/23/11 18:20 jec      began conversion from SMTemplate.c (02/20/07 rev)
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
#include "ES_Configure.h"
#include "ES_Framework.h"
#include "DunkButtonService.h"
#include "PIC32_Port_HAL.h"
#include "TestHarnessService.h"
#include "LeaderService.h"

/*----------------------------- Module Defines ----------------------------*/
#define DEBOUNCE_TIME 100
/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine.They should be functions
   relevant to the behavior of this state machine
*/

/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well.
// type of state variable should match htat of enum in header file
static DunkButtonState_t CurrentState;

static uint8_t LastDunkButtonState;

// with the introduction of Gen2, we need a module level Priority var as well
static uint8_t MyPriority;

/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
     InitTemplateFSM

 Parameters
     uint8_t : the priorty of this service

 Returns
     bool, false if error in initialization, true otherwise

 Description
     Saves away the priority, sets up the initial transition and does any
     other required initialization for this state machine
 Notes

 Author
     J. Edward Carryer, 10/23/11, 18:55
****************************************************************************/
bool InitDunkButton(uint8_t Priority)
{
  ES_Event_t ThisEvent;

  MyPriority = Priority;
  
  printf("initializing dunk button\r\n");
  
  // Init pin
  PortSetup_ConfigureDigitalInputs(_Port_B, _Pin_10);
  
  // set initial state to closed
  LastDunkButtonState = 0;
  
  // put us into the Initial PseudoState
  CurrentState = InitDunkButtonPState;
  // post the initial transition event
  ThisEvent.EventType = ES_INIT;
  if (ES_PostToService(MyPriority, ThisEvent) == true)
  {
    return true;
  }
  else
  {
    return false;
  }
}

/****************************************************************************
 Function
     PostTemplateFSM

 Parameters
     EF_Event_t ThisEvent , the event to post to the queue

 Returns
     boolean False if the Enqueue operation failed, True otherwise

 Description
     Posts an event to this state machine's queue
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:25
****************************************************************************/
bool PostDunkButton(ES_Event_t ThisEvent)
{
  return ES_PostToService(MyPriority, ThisEvent);
}

/****************************************************************************
 Function
    RunTemplateFSM

 Parameters
   ES_Event_t : the event to process

 Returns
   ES_Event_t, ES_NO_EVENT if no error ES_ERROR otherwise

 Description
   add your description here
 Notes
   uses nested switch/case to implement the machine.
 Author
   J. Edward Carryer, 01/15/12, 15:23
****************************************************************************/
ES_Event_t RunDunkButton(ES_Event_t ThisEvent)
{
  ES_Event_t ReturnEvent;
  ReturnEvent.EventType = ES_NO_EVENT; // assume no errors

  switch (CurrentState)
  {
    case InitDunkButtonPState:        // If current state is initial Psedudo State
    {
      if (ThisEvent.EventType == ES_INIT)    // only respond to ES_Init
      {
        // this is where you would put any actions associated with the
        // transition from the initial pseudo-state into the actual
        // initial state

        // now put the machine into the actual initial state
        CurrentState = DunkButtonIdle;
      }
    }
    break;

    case DunkButtonIdle:        // If current state is state one
    {
        if(ThisEvent.EventType == HW_BUTTONDOWN && ThisEvent.EventParam == 1){
            CurrentState = DunkButtonDown;
            ES_Timer_InitTimer(DUNK_BUTTON_TIMER, DEBOUNCE_TIME);
        }
    }
    break;
    
    case DunkButtonDown:
    {
        if (ThisEvent.EventType == ES_TIMEOUT && ThisEvent.EventParam == DUNK_BUTTON_TIMER){
            CurrentState = DunkButtonUp;
        }
        if(ThisEvent.EventType == HW_BUTTONUP){
            CurrentState = DunkButtonIdle;
        }
    }
    break;
    case DunkButtonUp:
    {
        if(ThisEvent.EventType == HW_BUTTONUP && ThisEvent.EventParam == 1){
            CurrentState = DunkButtonIdle;
            ES_Event_t newButtonPress;
            newButtonPress.EventType = HW_BUTTONPRESS;
            newButtonPress.EventParam = 1;
            PostTestHarnessService(newButtonPress);

            ES_Event_t enterDunkMode;
            enterDunkMode.EventType = HW_DUNKMODE;
            PostLeaderService(enterDunkMode);
            
        }
        
    }
    break;
    // repeat state pattern as required for other states
    default:
      ;
  }                                   // end switch on Current State
  return ReturnEvent;
}

/****************************************************************************
 Function
     QueryTemplateSM

 Parameters
     None

 Returns
     TemplateState_t The current state of the Template state machine

 Description
     returns the current state of the Template state machine
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:21
****************************************************************************/
DunkButtonState_t QueryDunkButtonState(void)
{
  return CurrentState;
}


bool CheckDunkButtonPress(void) {
    bool returnVal = false;
    uint8_t CurrentDunkButtonState = PORTBbits.RB10; // read current button value
    // if the button is currently 1, then it has been pressed down
    if(CurrentDunkButtonState != LastDunkButtonState){
        ES_Event_t newButtonEvent;
        newButtonEvent.EventParam = 1;
        returnVal = true;
        if(CurrentDunkButtonState == 1){
            newButtonEvent.EventType = HW_BUTTONDOWN;
        }else{
            newButtonEvent.EventType = HW_BUTTONUP;
        }
        PostDunkButton(newButtonEvent);
    }
    LastDunkButtonState = CurrentDunkButtonState;
    return returnVal;
}
/***************************************************************************
 private functions
 ***************************************************************************/

