/****************************************************************************
 Module
   LeaderService.c
 ****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
 */
#include "ES_Configure.h"
#include "ES_Framework.h"
#include "ES_DeferRecall.h"
#include "LeaderService.h"
#include "sys/attribs.h"
#include "PIC32_Port_HAL.h"
#include "EyeColor.h"
#include "ServoService.h"
#include "LoadButton.h"

/*----------------------------- Module Defines ----------------------------*/
#define DUNK_TIME 1500
#define ONE_SEC 1000
#define GAME_DURATION_IN_SEC 138
/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this service.They should be functions
   relevant to the behavior of this service
 */

/*---------------------------- Module Variables ---------------------------*/
// with the introduction of Gen2, we need a module level Priority variable
static uint8_t MyPriority;

static LeaderState_t CurrentState;

static ES_Event_t DeferralQueue[5 + 1];
static bool SentToggle = false;
static int SecondsPassed;

/*------------------------------ Module Code ------------------------------*/

/****************************************************************************
 Function
     InitLeaderService

 Parameters
     uint8_t : the priority of this service

 Returns
     bool, false if error in initialization, true otherwise

 Description
     Saves away the priority, and does any
     other required initialization for this service
 Notes

 Author
     J. Edward Carryer, 01/16/12, 10:00
 ****************************************************************************/
bool InitLeaderService(uint8_t Priority) {
    ES_Event_t ThisEvent;
    
    MyPriority = Priority;
    /********************************************
     in here you write your initialization code
     *******************************************/
    // initialize deferral queue for testing Deferal function
    ES_InitDeferralQueueWith(DeferralQueue, ARRAY_SIZE(DeferralQueue));
    
    printf("Initializing Leader Service...\r\n");
    InitEyeColors();
    TurnOffEyeColor();
    InitLoadButton();
    
    CurrentState = InitPState;
    
    SecondsPassed = 0;

    // post the initial transition event
    ThisEvent.EventType = ES_INIT;
    if (ES_PostToService(MyPriority, ThisEvent) == true) {
        return true;
    } else {
        return false;
    }
}

/****************************************************************************
 Function
     PostLeaderService

 Parameters
     EF_Event_t ThisEvent ,the event to post to the queue

 Returns
     bool false if the Enqueue operation failed, true otherwise

 Description
     Posts an event to this state machine's queue
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:25
 ****************************************************************************/
bool PostLeaderService(ES_Event_t ThisEvent) {
    return ES_PostToService(MyPriority, ThisEvent);
}

/****************************************************************************
 Function
    RunLeaderService

 Parameters
   ES_Event_t : the event to process

 Returns
   ES_Event, ES_NO_EVENT if no error ES_ERROR otherwise

 Description
   add your description here
 Notes

 Author
   J. Edward Carryer, 01/15/12, 15:23
 ****************************************************************************/
ES_Event_t RunLeaderService(ES_Event_t ThisEvent) {
    ES_Event_t ReturnEvent;
    ReturnEvent.EventType = ES_NO_EVENT; // assume no errors
    
    // Always check for game end timer timeout regardless of state
    if (ThisEvent.EventType == ES_TIMEOUT && ThisEvent.EventParam == GAME_DURATION_TIMER) {
        if (++SecondsPassed == GAME_DURATION_IN_SEC) {
            // GAME DURATION HAS PASSED
            CurrentState = GameEnd;
            ES_Event_t ToggleServoBrowL = {HW_TOGGLE_SERVO, SERVO_BROW_L};
            ES_Event_t ToggleServoBrowR = {HW_TOGGLE_SERVO, SERVO_BROW_R};
            PostServoService(ToggleServoBrowL);
            PostServoService(ToggleServoBrowR);
            TurnOffEyeColor();
            printf("NEED TO ADD 0x09 HANDLER IN FOLLOWER DUE TO 2 minute 18 second GAME DURATION TIMEOUT\r\n");
            printf("WILL NOW IGNORE ALL EVENTS DUE TO BEING IN GameEnd STATE\r\n");
            SPIOperate_SPI1_Send8(0x09); // Send GAME_END event to follower
            
        } else {
            ES_Timer_InitTimer(GAME_DURATION_TIMER, ONE_SEC);
        }
    }
    
    switch(CurrentState){
        case InitPState:
            if(ThisEvent.EventType == ES_INIT){
                CurrentState = GameIdle;
            }
        case GameIdle:
            if(ThisEvent.EventType == HW_LOADMODE || ThisEvent.EventType == HW_DUNKMODE){
                CurrentState = GameStart;
                ES_Timer_InitTimer(GAME_DURATION_TIMER, ONE_SEC);
                printf("sending GAME_START event to follower\r\n");
                SPIOperate_SPI1_Send8(0x10); // send GAME_START event to follower;
            }
            break;
            
        case GameStart:
        {
            if(ThisEvent.EventType == HW_BEACON){
                // Change eyebrow orientation
                ES_Event_t ToggleServoBrowL = {HW_TOGGLE_SERVO, SERVO_BROW_L};
                ES_Event_t ToggleServoBrowR = {HW_TOGGLE_SERVO, SERVO_BROW_R};
                PostServoService(ToggleServoBrowL);
                PostServoService(ToggleServoBrowR);
                
                printf("sending HW_BEACON event to follower\r\n");
                SPIOperate_SPI1_Send8(0x01); // send HW_Beacon event to follower
                
                CurrentState = Loading;
            }
            
        }
        break;
        case Loading:
        {
            // printf("in loading mode\r\n");
            switch(ThisEvent.EventType){
                case HW_LOADMODE:
                    printf("send HW_LOADMODE to follower\r\n");
                    SPIOperate_SPI1_Send8(0x03); // send HW_LOADMODE event to follower
                    break;
                case HW_DUNKMODE:
                    printf("send HW_DUNKMODE to follower\r\n");
                    SPIOperate_SPI1_Send8(0x02); // send HW_DUNKMODE event to follower
                    CurrentState = Dunking;
                    break;
                case HW_BEACON:
                    printf("sending HW_BEACON event to follower\r\n");
                    SPIOperate_SPI1_Send8(0x01); // send HW_BEACON event to follower
                    break;
                case HW_MISALIGNED:
                    printf("sending HW_MISALIGNED to follower\r\n");
                    SPIOperate_SPI1_Send8(0x08); // send HW_MISALIGNED event to follower
                    break;
            }
        }
        break;
        case Dunking:
        {
            ES_Event_t ToggleServoDunk = {HW_TOGGLE_SERVO, SERVO_DUNK};
            // printf("in dunking mode\r\n");
            switch(ThisEvent.EventType){
                
                case HW_LOADMODE:
                    printf("send HW_LOADMODE to follower\r\n");
                    SPIOperate_SPI1_Send8(0x03); // send HW_LOADMODE event to follower
                    break;
                case HW_TARGETREACHED:
                    printf("received HW TARGET REACHED\r\n");
                    // Release Servo
                    if(!SentToggle){
                        SentToggle = true;
                        PostServoService(ToggleServoDunk);
                    
                        // Init timer to allow for dunking
                        ES_Timer_InitTimer(DUNK_SERVICE_TIMER, DUNK_TIME);
                    }
                    break;
                case ES_TIMEOUT:
                    if (ThisEvent.EventParam == DUNK_SERVICE_TIMER) {
                        // Reset Servo
                        PostServoService(ToggleServoDunk);
                        // send HW_DUNKCOMPLETE event to follower
                        SPIOperate_SPI1_Send8(0x05); // send HW_DUNKCOMPLETE to follower;
                        // printf("sending DUNK COMPLETE to follower\r\n");
                    }
                    break;
                case HW_DUNKMODE:
                    // if dunk event
                    printf("send HW_DUNKMODE to follower\r\n");
                    SPIOperate_SPI1_Send8(0x02); // send HW_DUNKMODE to follower
                    SentToggle = false;
                    break;
                case HW_BEACON:
                    printf("sending HW_BEACON event to follower\r\n");
                    SPIOperate_SPI1_Send8(0x01); // send HW_BEACON event to follower;
                    break;
                case HW_MISALIGNED:
                    printf("send HW_MISALIGNED event to follower\r\n");
                    SPIOperate_SPI1_Send8(0x08); // send HW_MISALIGNED event to follower;
                    break;
            }
        }
        break;
        case GameEnd:
        {
            // ignore all events
        }
        break;
        
    }

//    if(ThisEvent.EventType == ES_RESPONSE_RECEIVED){
//        printf("received %x byte\r\n", ThisEvent.EventParam);
//    }

    return ReturnEvent;
}

LeaderState_t GetLeaderState(void){
    return CurrentState;
}
/***************************************************************************
 private functions
 ***************************************************************************/

/*------------------------------- Footnotes -------------------------------*/
/*------------------------------ End of file ------------------------------*/