/****************************************************************************
 Module
   SPIQueryLeaderService.c

 Revision
   1.0.1

 Description
   This is a file for implementing the SPI Leader Querying Service. It queries
 * the follower at a constant rate and will post events when a relevant command
 * is returned from the follower.

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 01/16/12 09:58 jec      began conversion from TemplateFSM.c
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
#include "ES_Configure.h"
#include "ES_Framework.h"
#include "SPIQueryLeaderService.h"
#include "PIC32_SPI_HAL.h"
#include "sys/attribs.h"
#include "PIC32_Port_HAL.h"
#include "LeaderService.h"

/*----------------------------- Module Defines ----------------------------*/
#define SPI_MODULE SPI_SPI1
#define QUERY_TIME 100
/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this service.They should be functions
   relevant to the behavior of this service
*/
static void InitSPI(void);
/*---------------------------- Module Variables ---------------------------*/
// with the introduction of Gen2, we need a module level Priority variable
static uint8_t MyPriority;
static uint8_t ByteReceived;
/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
     InitSPIQueryLeaderService

 Parameters
     uint8_t : the priorty of this service

 Returns
     bool, false if error in initialization, true otherwise

 Description
     Saves away the priority, and does any
     other required initialization for this service
 Notes

 Author
    Kathy Huang
****************************************************************************/
bool InitSPIQueryLeaderService(uint8_t Priority)
{
  ES_Event_t ThisEvent;

  MyPriority = Priority;
  
  InitSPI();
  printf("Initializing SPI Query Leader Service...\r\n");
  
  __builtin_enable_interrupts();
  /********************************************
   in here you write your initialization code
   *******************************************/
  // post the initial transition event
  ThisEvent.EventType = ES_INIT;
  if (ES_PostToService(MyPriority, ThisEvent) == true)
  {
    return true;
  }
  else
  {
    return false;
  }
}

/****************************************************************************
 Function
     PostSPIQueryLeaderService

 Parameters
     EF_Event_t ThisEvent ,the event to post to the queue

 Returns
     bool false if the Enqueue operation failed, true otherwise

 Description
     Posts an event to this state machine's queue
 Notes

 Author
     J. Edward Carryer, 10/23/11, 19:25
****************************************************************************/
bool PostSPIQueryLeaderService(ES_Event_t ThisEvent)
{
  return ES_PostToService(MyPriority, ThisEvent);
}

/****************************************************************************
 Function
    RunSPIQueryLeaderService

 Parameters
   ES_Event_t : the event to process

 Returns
   ES_Event, ES_NO_EVENT if no error ES_ERROR otherwise

 Description
   add your description here
 Notes

 Author
   J. Edward Carryer, 01/15/12, 15:23
****************************************************************************/
ES_Event_t RunSPIQueryLeaderService(ES_Event_t ThisEvent)
{
    ES_Event_t ReturnEvent;
    ReturnEvent.EventType = ES_NO_EVENT; // assume no errors
    if (ThisEvent.EventType == ES_INIT) {
        SPIOperate_SPI1_Send8(0xFF); // send no event to query follower
        ES_Timer_InitTimer(SPI_QUERY_TIMER, QUERY_TIME);
    }
    if (ThisEvent.EventType == ES_TIMEOUT) {
//        printf("query follower\r\n");
        SPIOperate_SPI1_Send8(0xFF);
        ES_Timer_InitTimer(SPI_QUERY_TIMER, QUERY_TIME);
    }
    return ReturnEvent;
}

/***************************************************************************
 private functions
 ***************************************************************************/
void __ISR(_SPI1_VECTOR, IPL5SOFT) _HW_SPI1Handler(void) {

    __builtin_disable_interrupts();

    do {
        IFS1CLR = _IFS1_SPI1RXIF_MASK;
        ByteReceived = SPI1BUF; // read new command
        if (ByteReceived == 0x04) { // only post target reached event
            ES_Event_t ReturnEvent;
            ReturnEvent.EventType = HW_TARGETREACHED;
            PostLeaderService(ReturnEvent);
        }
    } while (!SPI1STATbits.SPIRBE);
    
    __builtin_enable_interrupts();
   
}

static void InitSPI(void) {
    SPISetup_BasicConfig(SPI_MODULE);
    SPISetup_SetLeader(SPI_MODULE, SPI_SMP_MID);
    
    SPISetup_MapSSOutput(SPI_MODULE, SPI_RPA0); // 2, RA0
    SPISetup_MapSDOutput(SPI_MODULE, SPI_RPA2); // 9, RA2
    SPISetup_MapSDInput(SPI_MODULE, SPI_RPB8); // 17, RB8

    SPI1BUF; // Clear buffer

    SPI1STATbits.SPIROV = 0;
    SPISetEnhancedBuffer(SPI_MODULE, true);
    SPISetup_SetBitTime(SPI_MODULE, 1100); // BRG = 10
    SPISetup_SetXferWidth(SPI_MODULE, SPI_8BIT);
    SPISetup_SetActiveEdge(SPI_MODULE, SPI_FIRST_EDGE); // CKE = 0
    SPISetup_SetClockIdleState(SPI_MODULE, SPI_CLK_LO); // CKP = 1
    SPI1CONbits.SRXISEL = 0b01;

    IPC7bits.SPI1IP = 5;
    IFS1CLR = _IFS1_SPI1RXIF_MASK;
    IEC1SET = _IEC1_SPI1RXIE_MASK;

    SPISetup_EnableSPI(SPI_MODULE);
}
/*------------------------------- Footnotes -------------------------------*/
/*------------------------------ End of file ------------------------------*/

