/****************************************************************************
 Module
   UltrasonicService.c
/*----------------------------- Include Files -----------------------------*/
// This module
#include "../ProjectHeaders/UltrasonicService.h"

// debugging printf()
//#include "dbprintf.h"

// Hardware
#include <xc.h>
// #include <proc/p32mx170f256b.h>
#include <sys/attribs.h> // for ISR macors

// Event & Services Framework
#include "ES_Configure.h"
#include "ES_Framework.h"
#include "ES_DeferRecall.h"
#include "ES_ShortTimer.h"
#include "ES_Port.h"

/*----------------------------- Module Defines ----------------------------*/
#define ONE_SEC 1000
#define HALF_SEC 500
#define MILLI_SEC 1

#define ENTER_POST     ((MyPriority<<3)|0)
#define ENTER_RUN      ((MyPriority<<3)|1)
#define ENTER_TIMEOUT  ((MyPriority<<3)|2)

#define TEST_INT_POST
/*---------------------------- Module Functions ---------------------------*/
static void InitUltrasonic(void);
static void InitTMR1(void);
static void InitTMR2(void);
static void InitIC3(void);
/*---------------------------- Module Variables ---------------------------*/
// with the introduction of Gen2, we need a module level Priority variable
static uint8_t MyPriority;
// add a deferral queue for up to 3 pending deferrals +1 to allow for overhead
static ES_Event_t DeferralQueue[3 + 1];

static union counter {
    uint32_t all;

    struct {
        uint16_t buf;
        uint16_t rollover;
    } components;
} Counter;

static uint32_t CapturedTime, LastTime, PulseWidth;
static bool Pulsing, IsRisingEdge;

/*------------------------------ Module Code ------------------------------*/
bool InitUltrasonicService(uint8_t Priority) {
    ES_Event_t ThisEvent;

    MyPriority = Priority;

    Pulsing = false;
    IsRisingEdge = true;
    
    InitTMR1();
    InitTMR2();
    InitIC3();

    // post the initial transition event
    ThisEvent.EventType = ES_INIT;
    if (ES_PostToService(MyPriority, ThisEvent) == true) {
        return true;
    } else {
        return false;
    }
}

bool PostUltrasonicService(ES_Event_t ThisEvent) {
    return ES_PostToService(MyPriority, ThisEvent);
}

ES_Event_t RunUltrasonicService(ES_Event_t ThisEvent) {
    ES_Event_t ReturnEvent;
    ReturnEvent.EventType = ES_NO_EVENT; // assume no errors

    switch (ThisEvent.EventType) {
        case ES_INIT:
        {
//            ES_Timer_InitTimer(SERVICE0_TIMER, HALF_SEC);
        }
            break;
        case ES_TIMEOUT: // re-start timer & announce
        {
//            if (Pulsing) {
//                printf("STOPPING PULSE\r\n");
//                Pulsing = false;
//                LATBCLR = 1 << 9;
//                ES_Timer_InitTimer(SERVICE0_TIMER, HALF_SEC);
//            }

        }
            break;
        case EV_NEW_KEY: // announce
        {
            printf("EV_NEW_KEY received with -> %c <- in Service 0\r\n",
                    (char) ThisEvent.EventParam);
            Pulsing = true;
            LATBSET = 1 << 9;

            T1CONbits.ON = 1;
        }
            break;
        case EV_ECHO:
        {
            uint32_t PulseWidth = ThisEvent.EventParam;
            printf("EV_ECHO received with pulse width: %d\r\n", PulseWidth);
        }
        default:
        {
        }
            break;
    }

    return ReturnEvent;
}

/***************************************************************************
 private functions
 ***************************************************************************/

void InitUltrasonic(void) {
    ANSELBCLR = (1 << 5) | (1 << 9);
    TRISBSET = 1 << 5; // echo is input
    TRISBCLR = 1 << 9; // trigger is output
}

static void InitTMR1(void) {
    T1CONbits.ON = 0;

    T1CONbits.TCS = 0;
    T1CONbits.TCKPS = 0b00;
    TMR1 = 0;
    PR1 = 250;
    IFS0CLR = _IFS0_T1IF_MASK;
    IPC1bits.T1IP = 5;
    IEC0SET = _IEC0_T1IE_MASK;

    T1CONbits.ON = 1;
}


static void InitTMR2(void) {
    T2CONbits.ON = 0;

    T2CONbits.TCS = 0;
    T2CONbits.TCKPS = 0b011;
    TMR2 = 0;
    PR2 = 0xFFFF;
    IFS0CLR = _IFS0_T2IF_MASK;
    IPC2bits.T2IP = 6;
    IEC0SET = _IEC0_T2IE_MASK;

    T2CONbits.ON = 1;
}

void InitIC3(void) {
    IC3CONbits.ON = 0;

    IC3BUF = 0;
    IC3CONbits.FEDGE = 1; // rising edge
    IC3CONbits.C32 = 0;
    IC3CONbits.ICTMR = 1;
    IC3CONbits.ICI = 0;
    IC3CONbits.ICOV = 0;
    IC3CONbits.ICBNE = 0;
    IC3CONbits.SIDL = 0;
    IC3CONbits.ICM = 0b110; // every edge starting with FEDGE

    IC3R = 0b0001; // RPB5
    IEC0SET = _IEC0_IC3IE_MASK;
    IFS0CLR = _IFS0_IC3IF_MASK;
    INTCONbits.MVEC = 1;
    IPC3bits.IC3IP = 7;

    IC3CONbits.ON = 1;
}

/***************************************************************************
 interrupt service routines
 ***************************************************************************/

void __ISR(_TIMER_2_VECTOR, IPL6SOFT) _HW_Timer2Handler(void) {

    __builtin_disable_interrupts();
    if (IFS0bits.T2IF) {
        Counter.components.rollover += 1;
        IFS0CLR = _IFS0_T2IF_MASK;
    }

    __builtin_enable_interrupts();
}

void __ISR(_INPUT_CAPTURE_3_VECTOR, IPL7SOFT) _HW_IC3Handler(void) {
    __builtin_disable_interrupts();

    do {
        CapturedTime = IC3BUF;

        if (IFS0bits.T2IF && CapturedTime < 0x8000) {
            Counter.components.rollover += 1;
            IFS0CLR = _IFS0_T2IF_MASK;
        }
        Counter.components.buf = CapturedTime;

        PulseWidth = Counter.all - LastTime;
        LastTime = Counter.all;
        if (!IsRisingEdge) {
            ES_Event_t EchoEvent;
            EchoEvent.EventParam = PulseWidth;
            EchoEvent.EventType = EV_ECHO;
            LATBCLR = 1 << 9;
            PostUltrasonicService(EchoEvent);
        }
        
        IFS0CLR = _IFS0_IC3IF_MASK;

        IsRisingEdge = !IsRisingEdge;
        
    } while (IC3CONbits.ICBNE != 0);

    IFS0CLR = _IFS0_IC3IF_MASK;

    __builtin_enable_interrupts();
}

void __ISR(_TIMER_1_VECTOR, IPL5SOFT) _HW_Timer1Handler(void) {

    __builtin_disable_interrupts();
    if (IFS0bits.T1IF) {
        LATBCLR = 1 << 9;
        IFS0CLR = _IFS0_T1IF_MASK;
        TMR1 = 0;
        T1CONbits.ON = 0;
    }
    
    __builtin_enable_interrupts();
}

